@@ -37,6 +37,7 @@ import traceback
 import warnings
 import operator
 import ast
+import string
 
 # adding imports from further xonsh modules is discouraged to avoid circular
 # dependencies
@@ -274,6 +275,11 @@ class EnvPath(cabc.MutableSequence):
             self._l.insert(0 if front else len(self._l), data)
 
 
+@lazyobject
+def FORMATTER():
+    return string.Formatter()
+
+
 class DefaultNotGivenType(object):
     """Singleton for representing when no default value is given."""
 
@@ -610,6 +616,16 @@ def subexpr_before_unbalanced(expr, ltok, rtok):
     return subexpr
 
 
+@lazyobject
+def STARTING_WHITESPACE_RE():
+    return re.compile(r"^(\s*)")
+
+
+def starting_whitespace(s):
+    """Returns the whitespace at the start of a string"""
+    return STARTING_WHITESPACE_RE.match(s).group(1)
+
+
 def decode(s, encoding=None):
     encoding = encoding or DEFAULT_ENCODING
     return s.decode(encoding, "replace")
@@ -750,7 +766,7 @@ def _yield_accessible_unix_file_names(path):
         try:
             if file_.is_file() and os.access(file_.path, os.X_OK):
                 yield file_.name
-        except (FileNotFoundError, NotADirectoryError):
+        except OSError:
             # broken Symlink are neither dir not files
             pass
 
@@ -784,7 +800,11 @@ def _executables_in_windows(path):
                     yield fname
     else:
         for x in scandir(path):
-            if x.is_file():
+            try:
+                is_file = x.is_file()
+            except OSError:
+                continue
+            if is_file:
                 fname = x.name
             else:
                 continue
@@ -1163,6 +1183,11 @@ def to_logfile_opt(x):
     the filepath if it is a writable file or None if the filepath is not
     valid, informing the user on stderr about the invalid choice.
     """
+    superclass = pathlib.PurePath
+    if PYTHON_VERSION_INFO >= (3, 6, 0):
+        superclass = os.PathLike
+    if isinstance(x, superclass):
+        x = str(x)
     if is_logfile_opt(x):
         return x
     else:
@@ -1888,7 +1913,7 @@ def intensify_colors_on_win_setter(enable):
     environment variable.
     """
     enable = to_bool(enable)
-    if hasattr(builtins.__xonsh__, "shell"):
+    if builtins.__xonsh__.shell is not None:
         if hasattr(builtins.__xonsh__.shell.shell.styler, "style_name"):
             delattr(builtins.__xonsh__.shell.shell.styler, "style_name")
     return enable
@@ -1968,6 +1993,38 @@ def RE_COMPLETE_STRING():
     return re.compile(ptrn, re.DOTALL)
 
 
+def strip_simple_quotes(s):
+    """Gets rid of single quotes, double quotes, single triple quotes, and
+    single double quotes from a string, if present front and back of a string.
+    Otherwiswe, does nothing.
+    """
+    starts_single = s.startswith("'")
+    starts_double = s.startswith('"')
+    if not starts_single and not starts_double:
+        return s
+    elif starts_single:
+        ends_single = s.endswith("'")
+        if not ends_single:
+            return s
+        elif s.startswith("'''") and s.endswith("'''") and len(s) >= 6:
+            return s[3:-3]
+        elif len(s) >= 2:
+            return s[1:-1]
+        else:
+            return s
+    else:
+        # starts double
+        ends_double = s.endswith('"')
+        if not ends_double:
+            return s
+        elif s.startswith('"""') and s.endswith('"""') and len(s) >= 6:
+            return s[3:-3]
+        elif len(s) >= 2:
+            return s[1:-1]
+        else:
+            return s
+
+
 def check_for_partial_string(x):
     """Returns the starting index (inclusive), ending index (exclusive), and
     starting quote string of the most recent Python string found in the input.
@@ -2283,6 +2340,9 @@ def columnize(elems, width=80, newline="\n"):
     return lines
 
 
+ALIAS_KWARG_NAMES = frozenset(["args", "stdin", "stdout", "stderr", "spec", "stack"])
+
+
 def unthreadable(f):
     """Decorator that specifies that a callable alias should be run only
     on the main thread process. This is often needed for debuggers and
